/* Matthew A Hammer */
/* <hammer@tti-c.org */

#include "freeset.h"
#include <stdlib.h>
#include <assert.h>

/* A freeset is implemented as an array of freelists where the ith
 * slot of the array corresponds to a block-size of i words for the
 * ith freelist, stored there.
 *
 * The role of the freeset is to initialize and then multiplex into
 * this array of freelists.
 */

static unsigned int
nwords_for_nbytes(unsigned int nbytes) {
  return ((nbytes / sizeof(void*)) + (nbytes % sizeof(void*) ? 1 : 0));
}

static freelist_t*
freelist_for_nwords(freeset_t* freeset, unsigned int nwords) {
  int i = nwords - 1;
  assert(i < freeset->freelistc);
  return (&((freeset)->freelistv[i]));
}

void  freeset_init(freeset_t* freeset, unsigned int max_block_size) {
  int freelistc = nwords_for_nbytes(max_block_size);
  assert(freelistc < 256);  
  freeset->freelistc = freelistc;
  freeset->freelistv = malloc(sizeof(freelist_t) * freelistc);

  {int i; for(i = 1; i < freelistc; i++) {
    freelist_t* freelist = freelist_for_nwords(freeset, i);
    freelist_init(freelist, i * sizeof(void*));
  }}
}

void* freeset_pop(freeset_t* freeset, unsigned int size) {
  if(!size) {
    return NULL;
  }
  else {
    int words = nwords_for_nbytes(size);
    freelist_t* freelist = freelist_for_nwords(freeset, words);
    return freelist_pop(freelist);
  }
}

void  freeset_push(freeset_t* freeset, unsigned int size, void* ptr) {
  if(!size || !ptr) {
    return;
  }
  else {
    int words = nwords_for_nbytes(size);
    freelist_t* freelist = freelist_for_nwords(freeset, words);
    freelist_push(freelist, ptr);
  }
}
