#include <stdlib.h>
#include <assert.h>
#include <stdio.h>

/* Get the header if we don't already have it. */
#ifndef __HASHTBL_H__
#define __HASHTBL_C__
#include "hashtbl.h"
#undef __HASHTBL_C__
#endif

#define GROWTH_FACTOR  2
#define SHRINK_FACTOR  4
#define INIT_SIZE      7

HASHTBL_FUN
HASHTBL(_t)* HASHTBL(_init)(HASHTBL(_t)* h, int size) {
  int i;
  
  h->size  = size;
  h->num   = 0;
  h->table = HASHTBL_MALLOC_ARRAY(h->size * sizeof(doublink_head_t));
  
  for(i = 0; i < h->size; i++) {
    doublink_init(&(h->table[i]));
  }
  
  return h;
}

static HASHTBL_FUN
void HASHTBL(_move)(HASHTBL(_t)* src, HASHTBL(_t)* dst) {
  int bucket;
  
  for(bucket = 0; bucket < src->size; bucket++) {
    doublink_head_t* head = &(src->table[bucket]);
    doublink_node_t* node = doublink_seq_remove(head, NULL);
        
    /* Move each node from source chain into corresponding destination
       chain (this differs based on the new modulus) -- we preserve
       monotonicity in the final ordering by moving left-to-right in
       the source lists and pushing to the back (at the far-right) of
       each destination list. */
    while(node) {
      doublink_node_t* next = node->next;
      hash_val_t hash = HASHTBL_HASH_FUN(node);
      
      /* Move the node to the destination table */
      doublink_push_last(& dst->table[hash % dst->size], node);
      dst->num++;
      node = next;
    }

    doublink_init(&(src->table[bucket]));
  }
  src->num = 0;
}

static HASHTBL_FUN
void HASHTBL(_resize)(HASHTBL(_t)* h, int newsize) {
  HASHTBL(_t) hh;
  HASHTBL(_init)(&hh, newsize);
  HASHTBL(_move)(h, &hh);  
  HASHTBL_FREE_ARRAY(h->size * sizeof(doublink_head_t), h->table);
  *h = hh;
}

HASHTBL_FUN void
HASHTBL(_insert) (HASHTBL(_t)* h, doublink_node_t* node) {
  if(h->num + 1 > h->size * GROWTH_FACTOR) {
    HASHTBL(_resize)(h, (h->size > 0) ? h->size * GROWTH_FACTOR : INIT_SIZE);
  }
  hash_val_t hash = HASHTBL_HASH_FUN(node);
  doublink_head_t* head = &(h->table[hash % h->size]);
  doublink_push_last(head, node);
  h->num ++;  
}

HASHTBL_FUN void
HASHTBL(_remove) (HASHTBL(_t)* h, doublink_node_t* node) {
  doublink_remove(node);
  h->num --;

  if(h->num == 0) {
    /* This ensures that empty hashtables take no space. */
    HASHTBL(_resize)(h, 0);
  }
  else if(h->num <= h->size / SHRINK_FACTOR) {
    HASHTBL(_resize)(h, h->size / SHRINK_FACTOR);
  }  
}

HASHTBL_FUN doublink_node_t*
HASHTBL(_lookup) (HASHTBL(_t)* h, hash_val_t hash) {
  if(h->size)
    return doublink_get_first(&(h->table[hash % h->size]));
  else
    return NULL;
}
