/* Matthew Hammer  */
/* <hammer@tti-c.org> */

#define __HASHTBL_H__

#include "adt_directives.h"
#include "doublink.h"
#include "hash.h"

/* Hash Table - a hashtable implementation which uses doubly-linked
   lists at each slot to cope with collisions. */

/* The user defines the following before including this header:
 *
 * -- HASHTBL(suffix) as prefix##suffix for some user prefix.
 * -- HASHTBL_HASH_FUN(node) to be the function for hashing nodes.
 * -- HASHTBL_MALLOC_ARRAY(sz)
 * -- HASHTBL_FREE_ARRAY(sz, ptr)
 */

/* Setup default symbol-gen macro */
#ifndef HASHTBL
#define HASHTBL(suffix) hashtbl##suffix
#endif

/* Abstract-type for the hashtbl */
ABSTYPE(HASHTBL(_s), HASHTBL(_t));

/* -- API -- */
HASHTBL_FUN HASHTBL(_t)*     HASHTBL(_init)    (HASHTBL(_t)* h, int size);
HASHTBL_FUN void             HASHTBL(_insert)  (HASHTBL(_t)* h, doublink_node_t* node);
HASHTBL_FUN void             HASHTBL(_remove)  (HASHTBL(_t)* h, doublink_node_t* node);
HASHTBL_FUN doublink_node_t* HASHTBL(_lookup)  (HASHTBL(_t)* h, hash_val_t hash);

#include "hashtbl_impl.h"
