/* Matthew Hammer */
/* <hammer@tti-c.org */

/* See: <totalorder.h> for documentation about the data-structure/API
   that this set of tests applies to. */

#include <assert.h>
#include <stdio.h>
#include "totalorder.h"

typedef unsigned long long mylong_t;

static to_label_t check_order_count = 0;
static to_data_t check_order(to_data_t prev, to_node_t* n, to_data_t cur) {
  if(prev) {
    to_node_t* prev_node = (to_node_t*) prev;

    /*
    to_label_t prev_label = prev_node->label;
    to_label_t cur_label = n->label;    
    printf("check_order: %10lu %10lu\n", prev_label, cur_label);
    */
    
    assert(totalorder_compare(prev_node, n) < 0);
    check_order_count ++;
  }
  return (to_data_t) n;
}

static void validate(to_node_t* first, to_node_t* last, int count) {
  printf("Verifying invariants...\n");

  /* Check */
  printf("  Checking (all) 1st-level nodes... "); fflush(stdout);
  totalorder__check1(first->parent);
  printf(" done.\n");
  
  /* Check */
  check_order_count = 0;
  printf("  Checking (all) 2nd-level nodes... "); fflush(stdout);
  totalorder_iterate(first, last, check_order, 0);  
  printf(" done. (Did %llu check(s) for order on %llu nodes)\n",
         (mylong_t) check_order_count,
         (mylong_t) count);
}

int main(int argc, char** argv) {
  to_node_t* first = totalorder_create();
  to_node_t* last = totalorder_insert(first);
  to_label_t inserts = 1;
  to_label_t count0 = 2;
  
  printf("sizeof(to_label_t)=%d\n", sizeof(to_label_t));
  printf("sizeof(to_data_t)=%d\n", sizeof(to_data_t));
  printf("sizeof(to_node1_t)=%d\n", sizeof(to_node1_t));
  printf("sizeof(to_node2_t)=%d\n", sizeof(to_node2_t));
  printf("TO_LIST_SIZE=%d\n", TO_LIST_SIZE);
  
  while(1){
    to_node_t* n = first;
    to_label_t count = count0;
    int summaries = 0;
    

    printf("\n-------------------------------------------------\n");
    printf("Doing INSERTs ...\n");

    while(1) {
      /* every so often, check to make sure that the structure is
         valid. */
      if(!((count-count0) % (1 << 20))) {
        validate(first, last, count);
        
        ++ summaries;
        if(summaries >= 10) break;
        
        printf("\n%10s %10s %10s %6s %6s %6s %10s %10s \n",
               "INSERTs",
               "count2", "count1",
               "size2", "size1",
               "size-T", "cost", "cost/ins");
      }
         

      n = totalorder_insert(n);
      ++ count;
      ++ inserts;
         
      /* print stats every so often */
      if(!((count-count0) % (1 << 16))) {
        /* estimate the number of top-level nodes */
        to_label_t count_1 = count / TO_LIST_SIZE;
        to_label_t size_1 = (((mylong_t) count_1) * sizeof(to_node1_t)) >> 20;
        to_label_t size_2 = (((mylong_t) count) * sizeof(to_node_t)) >> 20;
        /*
          print the # of (2nd-level) nodes, the estimated size, in MB,
          of the nodes created so far, the cost of relabeling so far,
          and the ammortized cost of relabeling so far.
        */        
        printf("%10llu %10llu %10llu %5llum %5llum %5llum %10llu %10f\n",
               (mylong_t) inserts,
               (mylong_t) count,
               (mylong_t) count_1,
               (mylong_t) size_2,
               (mylong_t) size_1, (mylong_t) (size_2 + size_1),
               (mylong_t) totalorder_relabel_count,
               (double) totalorder_relabel_count / (double) inserts);
      }
    }

    printf("Doing REMOVE ...\n");
    totalorder_remove(first, last);
  }
  return 0;
}
