/* Matthew Hammer  */
/* <hammer@tti-c.org> */

#define __VECTOR_H__

#include "adt_directives.h"
#include "hash.h"

/* Vector - arrays with associated capacity (maximum number of
   elements before a resize) and length (current number of elements)
   that automatically resize when necessary.  Operations push() and
   pop() treat the end of the vector like a stack.  Operations begin()
   and end() allow iteration.
*/

/* The user defines the following before including this header:
 *
 * -- VECTOR(suffix): as prefix##suffix for some user prefix.
 * -- VECTOR_VAL_TYPEDEF: to be the type of the elements in the vector.
 */

/* Setup default symbol-gen macro */
#ifndef VECTOR
#define VECTOR(suffix) vector##suffix
#endif

/* Setup the element-type */
#ifndef VECTOR_VAL_TYPEDEF
#define VECTOR_VAL_TYPEDEF unsigned int
#endif
typedef VECTOR_VAL_TYPEDEF VECTOR(_val_t);

/* Abstract-type: */
ABSTYPE(VECTOR(_s), VECTOR(_t));
typedef unsigned int VECTOR(_num_t);

/* -- API -- */
VECTOR_FUN VECTOR(_t)*     VECTOR(_init)(VECTOR(_t)* v, VECTOR(_num_t) size);
VECTOR_FUN void            VECTOR(_cleanup)(VECTOR(_t)* v);
VECTOR_FUN VECTOR(_t)*     VECTOR(_from_array)(VECTOR(_t)* v, int valc, VECTOR(_val_t)* vals);

VECTOR_FUN void            VECTOR(_push)(VECTOR(_t)* v, VECTOR(_val_t) val);
VECTOR_FUN VECTOR(_val_t)  VECTOR(_pop)(VECTOR(_t)* v);
VECTOR_FUN int             VECTOR(_isempty)(VECTOR(_t)* v);

VECTOR_FUN hash_val_t      VECTOR(_hash)(VECTOR(_t)* v);
VECTOR_FUN int             VECTOR(_equals)(VECTOR(_t)* v, VECTOR(_t)* u);

VECTOR_FUN VECTOR(_val_t)* VECTOR(_begin)(VECTOR(_t)* v);
VECTOR_FUN VECTOR(_val_t)* VECTOR(_end)(VECTOR(_t)* v);

VECTOR_FUN VECTOR(_t)*     VECTOR(_copy)(VECTOR(_t)* srcv, VECTOR(_t)* dstv);
VECTOR_FUN VECTOR(_t)*     VECTOR(_copy_shallow)(VECTOR(_t)* srcv, VECTOR(_t)* dstv);
VECTOR_FUN VECTOR(_t)*     VECTOR(_copy_deep)(VECTOR(_t)* srcv, VECTOR(_t)* dstv);

#include "vector_impl.h"
