#ifndef __ACTK_TYPES__
#define __ACTK_TYPES__

#include "actk_directives.h"
#include "../adt/hash.h"

#define ACTK_STRUCT(structname, typename) \
  struct structname;\
  typedef struct structname typename;

/**
 * \addtogroup actk_public
 * @{
 */

typedef void* actk_val_t;
typedef void* actk_key_t; 
typedef unsigned char actk_byte_t;

typedef enum actk_flags_e {
  ACTK_LOG_STDERR       = 0x01,     /* global, default:off */
  ACTK_LOG_FILE         = 0x02,     /* global, default:off */
  ACTK_LOG_INTERVALS    = 0x04,     /* global, default:off */
  ACTK_RM_INTERVALS     = 0x08,     /* global, default:off */
  ACTK_PRINT_PROP_STATS = 0x10,     /* global, default:off */
  ACTK_TAILRECURSION    = 0x20,     /* global, default:on  */
  ACTK_BOOKKEEPING      = 0x40,     /* global, default:on  */
} actk_flags_t;

ACTK_STRUCT(actk_stats_s, actk_stats_t);

/** \seealso actk_init */
ACTK_STRUCT(actk_state_s, actk_state_t);
ACTK_STRUCT(actk_fun_s, actk_fun_t);
ACTK_STRUCT(actk_closure_s, actk_closure_t);
ACTK_STRUCT(actk_modref_s, actk_modref_t);

/* @} */

/**
 * \addtogroup actk_private
 * @{
 */

ACTK_STRUCT(actk_funarg_s, actk_funarg_t);
ACTK_STRUCT(actk_funsig_s, actk_funsig_t);
ACTK_STRUCT(actk_interval_s, actk_interval_t);
ACTK_STRUCT(actk_recctx_s, actk_recctx_t);
ACTK_STRUCT(actk_chpctx_s, actk_chpctx_t);

typedef enum actk_argdesc_e {
  ACTK_FUN_ARG_READ_LOC_OR_VAL,
  ACTK_FUN_ARG_CONST_VAL,
  ACTK_FUN_ARG_CONST_DST,
  ACTK_FUN_ARG_LIFTD_DST,
  ACTK_FUN_ARG_ENVIR_VAL,
} actk_argdesc_t;

struct actk_funarg_s {
  const char* name;
  actk_argdesc_t desc;
};

typedef enum actk_funflags_e {
  ACTK_TAILRECURSIVE = 1,
  ACTK_USEMEMOTABLE = 2,
  ACTK_USEALLOCTABLE = 4,
} actk_funflags_t;

struct actk_funsig_s {
  const char* name;
  const char* description;
  int argc;
  actk_funarg_t* args;
  int closure_bytec;
  int recctx_bytec;
};

typedef void (*actk_funptr_t) (actk_state_t* st);

struct actk_fun_s {
  actk_funsig_t* sig;
  actk_funptr_t funptr;
  actk_funflags_t flags;
  IF_DEBUG( int next_num_closure );
  IF_DEBUG( int next_num_recctx );
};

/* @} */

#endif
