#ifndef __ALLOCATION2_H__
#define __ALLOCATION2_H__

#include "actk_types.h"

/**
 * \defgroup actk_private_allocation Allocation
 * \ingroup actk_private
 * @{
 */

ACTK_STRUCT(actk_alloc_hd1_s, actk_alloc_hd1_t);
ACTK_STRUCT(actk_alloc_hd2_s, actk_alloc_hd2_t);
ACTK_STRUCT(actk_alloc_hd3_s, actk_alloc_hd3_t);

/* actk_alloc_new(): allocates size bytes of space and returns a
 * pointer to it.  if the keyc is zero, then the corresponding
 * allocation will be done without keys.  Moreover, this will try to
 * reuse or steal if keys are provided.  These situations are
 * conditioned on the current state, including the current closure,
 * etc.  They will always succeed in returning something unless the
 * entire system is out of memory.
 *
 * An allocation is fresh when it is not reused or stolen.  Fresh
 * allocations are always zeroed.  Reused allocations are never
 * zeroed.
 */
void* actk_alloc_new(actk_state_t* st, unsigned int size, int keyc, void** keyv);

/* Make an allocation into garbage -- to be used only at the toplevel. */
void actk_alloc_garb(actk_state_t* st, void* ptr);

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/* Make all allocations done by a given closure into garbage -- more
   efficient than calling garb() for each one. */
void actk_alloc_garball(actk_state_t* st, actk_closure_t* closure);

/* Transfer ownership of closure0's allocations to closure1 */
void actk_alloc_adoptall(actk_state_t* st, actk_closure_t* closure0, actk_closure_t* closure1);

/* Free all garbage -- all this garbage becomes available for "fresh"
   allocations.  Note this is only sound after change-propagation is
   completed and specifically, after none of the garbage is reachable
   from any live allocations. */
void actk_alloc_freeall(actk_state_t* st);

/* Update the maxlive stats as the current difference between the
   total allocated and the total freed. */
void actk_alloc_dostats(actk_state_t* st);

/* Given a Header-(3), hash it.  This is used by the allocation-table
   implementation. */
hash_val_t actk_alloc_hash(actk_alloc_hd3_t* hd3);

/* @} */

#endif
