#ifndef __CLOSURE_H__
#define __CLOSURE_H__

#include "actk_types.h"
#include "timestamp.h"

/**
 * \defgroup actk_private_closure Closure
 * \ingroup actk_private
 * @{
 */

typedef unsigned int actk_argvmask_t;

typedef enum actk_closure_enter_flags_e {
  ACTK_CLOSURE_ENTER_NEVER,
  ACTK_CLOSURE_ENTER_AUTO,
  ACTK_CLOSURE_ENTER_PROPAGATE,
} actk_closure_enter_flags_t;

int
actk__closure_compare(actk_closure_t* closure1, actk_closure_t* closure2);

actk_timestamp_t*
actk__closure_get_end(actk_state_t* st, actk_closure_t* closure);

actk_closure_t*
actk__closure_setpos(actk_closure_t* closure, int pos);

void
actk__closure_sizes(actk_fun_t* fun, int* closure_bytec, int* recctx_bytec);

actk_closure_t*
actk__closure_new(actk_state_t* st,
                  actk_fun_t* fun,
                  actk_recctx_t* recctx,
                  actk_argvmask_t rkeymask,
                  actk_argvmask_t readmask,
                  actk_val_t* args);

actk_recctx_t*
actk__recctx_new(actk_state_t* st,
                 actk_fun_t* fun,
                 actk_argvmask_t rkeymask,
                 actk_val_t* args);


void actk__closure_subscribe(actk_state_t* st, actk_closure_t* closure);
void actk__closure_unsubscribe(actk_state_t* st, actk_closure_t* closure);

void actk__closure_enqueue(actk_state_t* st, actk_closure_t* closure);

void actk__closure_init(actk_state_t* st, actk_closure_t* closure);
void actk__closure_enter(actk_state_t* st, actk_closure_t* closure, actk_closure_enter_flags_t flags);
actk_closure_t* actk__closure_wakeup(actk_state_t* st, actk_closure_t* closure);

hash_val_t actk__closure_hashval(actk_closure_t* closure);
int        actk__closure_equals(actk_closure_t* closure_1, actk_closure_t* closure_2);

void actk__closure_replace(actk_state_t* st, actk_closure_t* closure_old, actk_closure_t* closure_new);

int actk__closure_isuptodate(actk_state_t* st, actk_closure_t* closure);

actk_state_t* actk__closure_invalidate(actk_state_t* st, actk_timestamp_t* timestamp, void* data);
void actk__recctx_cleanup(actk_state_t* st, actk_recctx_t* recctx);

int actk__closure_verify(actk_state_t* st, actk_closure_t* closure); /* <-- for debugging */

/* @} */

#endif
