#ifndef __CLOSURE_IMPL__
#define __CLOSURE_IMPL__

#include "closure.h"
#include "memotbl.h"
#include "alloctbl.h"
#include "readlist.h"
#include "readqueue.h"
#include "../adt/doublink.h"
#include "modref_impl.h"

/**
 * \addtogroup actk_private_closure
 * @{
 */

typedef enum actk_closure_color_e {
  ACTK_CLOSURE_NONE,  /* NONE  : not fully-initialized/valid yet. */
  ACTK_CLOSURE_WHITE, /* WHITE : initialized, valid */
  ACTK_CLOSURE_GREY,  /* GREY  : on change-prop queue */
  ACTK_CLOSURE_BLACK, /* BLACK : invalid */
} actk_closure_color_t;

struct actk_recctx_s {
  actk_fun_t* fun;
  memotbl_t memotbl;
  alloctbl_t alloctbl;
  actk_timestamp_t* end;
  IF_DEBUG( int next_num_closure );
  IF_DEBUG( int num );
};

struct actk_closure_s {
  doublink_node_t memotbl_node;
  doublink_head_t alloc_set;
  actk_recctx_t* recctx;
  actk_interval_t interval;  
  actk_closure_color_t color;
  readqueue_pos_t readqueue_pos;
  IF_DEBUG( int num );
  IF_DEBUG( int num_recur );
};

typedef struct actk_closure_read_s {
  actk_modref_t* modref;
  actk_val_t val;
  readlist_handle_t handle;
} actk_closure_read_t;

typedef union actk_closure_slot_u {
  actk_closure_read_t read;
  actk_modref_t write;
  actk_val_t    const_val;
  actk_val_t    envir_val;
} actk_closure_slot_t;

/*@}*/

#endif
