#include "heap_impl.h"
#include "stats.h"
#include "../adt/freeset.h"

#include <stdlib.h>
#include <assert.h>

/* Statistics table hooked to these memory routines. */
static actk_stats_t* stats = NULL;

#define USE_FREESET 0

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Libc wrappers */

static actk_libc_block_hdr_t*
hdr_of_ptr(void* ptr) {
  actk_libc_block_hdr_t* hdr = ptr;
  return (hdr - 1);
}

static void*
ptr_of_hdr(actk_libc_block_hdr_t* hdr) {
  return (void*) (hdr + 1);
}

void* actk_libc_malloc(int size) {
  if(!size) {
    return NULL;
  }
  else {
    actk_libc_block_hdr_t* hdr =
      malloc(sizeof(actk_libc_block_hdr_t) + size);
    if(!hdr) {
      perror(__FUNCTION__);
      exit(EXIT_FAILURE);
    }
    hdr->size = size;
    if(stats){
      actk__stats_add(stats, TOTAL_MALLOC, size);
    }
    return ptr_of_hdr(hdr);
  }
}

int actk_libc_sizeof(void* ptr) {
  actk_libc_block_hdr_t* hdr = hdr_of_ptr(ptr);
  return hdr->size;
}

void actk_libc_free(void* ptr) {
  if(ptr) {
    actk_libc_block_hdr_t* hdr = hdr_of_ptr(ptr);
    if(stats){actk__stats_add(stats, TOTAL_FREE, hdr->size);}
    free(hdr);        
  }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Freeset wrappers */

#define DEFAULT_MAX_BLOCK_SIZE 128

#if USE_FREESET
static freeset_t global_freeset_ = {0,0,NULL};
freeset_t* global_freeset = &global_freeset_;
#endif

void actk_freeset_init() {
#if USE_FREESET
  freeset_init(global_freeset, 128);
#endif
}

void* actk_freeset_malloc(int size) {
  if(stats){actk__stats_add(stats, TOTAL_MALLOC, size);}
#if USE_FREESET
  return freeset_pop(global_freeset, size);
#else
  return malloc(size);
#endif
}

void actk_freeset_free(int size, void* ptr) {
  if(stats){actk__stats_add(stats, TOTAL_FREE, size);}
#if USE_FREESET
  freeset_push(global_freeset, size, ptr);
#else
  free(ptr);
#endif
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Misc */

void actk_heap_set_stats(actk_stats_t* stats_) {
  assert(!stats);
  assert(stats_);
  stats = stats_;
}
