/* Matthew Hammer <hammer@tti-c.org> */
/* Sat Oct 14 13:13:14 CDT 2006 */

#include "logging.h"
#include "state_impl.h"

#include <stdio.h>
#include <time.h>
#include <stdarg.h>
#include <libgen.h>

int actk__log_major_num = 0;
int actk__log_minor_num = 0;
int actk__log_line_num = 0;

static int
logcatf_stamp(FILE* f,
             const char* file, const char* fun, int line)
{
  return fprintf(f, "%15d-%d.%d: %-30s: %3d: ",
                 actk__log_line_num,
                 actk__log_major_num,
                 actk__log_minor_num,
                 fun,
                 line);
}

int logcatf_full(actk_state_t* st,
                 const char* file,
                 const char* fun,
                 int line,
                 const char* fmt, ...)
{
  int bytes = 0;
  va_list ap;            
  static FILE* logfile = NULL;
  int i;
  FILE* outv[2];
  int outc = 0;
    
  actk__log_line_num++;
  
  if(!logfile && (st->flags & ACTK_LOG_FILE))
    logfile = fopen("actk.log", "w+");

  if(st->flags & ACTK_LOG_FILE)
    outv[outc++] = logfile;

  if(st->flags & ACTK_LOG_STDERR)
    outv[outc++] = stderr;

  for(i = 0; i < outc; i++) {
    bytes += logcatf_stamp(outv[i], file, fun, line);
    va_start(ap, fmt);
    bytes += vfprintf(outv[i], fmt, ap);
    va_end(ap);
    fflush(outv[i]);
  }
  return bytes;
}
