#define HASHTBL_INCSRC  1
#include "memotbl.h"
#undef HASHTBL_INCSRC

#include "logging.h"
#include "closure.h"
#include "closure_impl.h"
#include "state_impl.h"
#include "actk_debug.h"

actk_closure_t*
actk_memo_match(actk_state_t* st, memotbl_t* memotbl, actk_closure_t* closure) {
  actk_closure_t* match = NULL;
  
  logcatf(st, "trying to match " DBSTR_CLOSURE, DBVAS_CLOSURE(closure));
  logcatf(st, "  in " DBSTR_STATE, DBVAS_STATE(st));
  logcatf(st, "  hash is %x", actk__closure_hashval(closure));
  
  {
    hash_val_t hash = actk__closure_hashval(closure);
    doublink_node_t* first = memotbl_lookup(memotbl, hash);
    doublink_node_t* node = first;
    
    while(node) {
      actk_closure_t* closure0 = (actk_closure_t*) node;
      
      /* CHECK: closure to reuse is after now */
      int time_compare_start = totalorder_compare(st->now,
                                                  closure0->interval.start);

      /* CHECK: closure to reuse is before chpctx end */
      int time_compare_end   = totalorder_compare(closure0->interval.start,
                                                  st->chpctx->end);

      if(node->next == first)
        node = NULL;
      else
        node = node->next;
      
      logcatf(st, "-- considering : " DBSTR_CLOSURE, DBVAS_CLOSURE(closure0));
      
      if(time_compare_start > 0) {
        logcatf(st, "-- -- start-time is too early");
        continue;
      }

      if(time_compare_end >= 0) {
        logcatf(st, "-- -- start-time is too late");
        continue;
      }

      if(actk__closure_equals(closure0, closure)) {
        logcatf(st, "-- -- success.");
        match = closure0;
        break;
      }
      else {
        logcatf(st, "-- -- closures aren't equal");
      }
    }
    
    if(match) {
      logcatf(st, "memo-matched : " DBSTR_CLOSURE, DBVAS_CLOSURE(closure));
      logcatf(st, "   with prev : " DBSTR_CLOSURE, DBVAS_CLOSURE(match));
    }
    else {
      logcatf(st, "no match for : " DBSTR_CLOSURE, DBVAS_CLOSURE(closure));
      logcatf(st, "     hash is : %x", actk__closure_hashval(closure));
    }
  }
  
  return match;
}
