#ifndef __STATE_IMPL__
#define __STATE_IMPL__

#include "actk_directives.h"
#include "actk_types.h"
#include "timestamp.h"
#include "readqueue.h"
#include "state.h"
#include "../adt/doublink.h"

/* State: */
struct actk_state_s {
  /* The toplevel closure is special in that it is never "woken-up" or
     "memo-matched" during change-propagation, instead it represents
     the "meta"-computation and is the currently-running closure
     before-and-after the initial-run and before-and-after
     change-propagation. */
  actk_closure_t* toplevel;

  /* The timestamp that is considered to be the "current-time" */
  actk_timestamp_t* now;
  
  /* current change-propagation context, may be NULL if in
     initial-run. */
  actk_chpctx_t* chpctx;
  
  /* current closure being executed */
  /* TODO: Should be exposed for macro API. */
  /* TODO: Also need: closure->recctx, recctx->fun */
  actk_closure_t* closure;

  /* for tail-call optimization, this flag controls whether or not the
     closure should loop (corresponding to a recursive function call
     at the tail of the current function) or terminate.  It's set by
     actk__closure_enter(), and cleared again after the loop is
     entered (or reentered).  */
  /* TODO: Should be exposed for macro API. */
  int fun_loop_on;

  /* current keys for allocation, the keys themselves are assumed to
     be allocated statically (e.g., on the stack). */
  unsigned int alloc_keyc;
  void** alloc_keyv;  

  actk_byte_t* alloc_begin;
  actk_byte_t* alloc_end;
  doublink_head_t garb_set;
  
  /* statistics */
  /* TODO: should be exposed for testing code. */
  actk_stats_t *stats;

  /* "global" flags */
  actk_flags_t flags;
};

/* Change-propagation Context: */
struct actk_chpctx_s {
  actk_timestamp_t* end;
  readqueue_t* queue;
};

#endif
