#include "stats.h"
#include <assert.h>

static const actk_stats_stat_t actk_stats[] =
  {{ ACTK_STATS_CLOSURE_NEW,        "closure-new",         ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_CLOSURE_INIT,       "closure-init",        ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_CLOSURE_ENTER,      "closure-enter",       ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_CLOSURE_REPLACE,    "closure-replace",     ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_CLOSURE_INVALIDATE, "closure-invalidate",  ACTK_STATS_INT_SUM, 0},
   
   { ACTK_STATS_RECCTX_NEW,         "recctx-new",          ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_RECCTX_FRESH,       "recctx-fresh",        ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_RECCTX_REUSE,       "recctx-reuse",        ACTK_STATS_INT_SUM, 0},   

   { ACTK_STATS_TOTAL_MALLOC,       "total-malloc",        ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_TOTAL_FREE,         "total-free",          ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_TOTAL_MAXLIVE,      "total-maxlive",       ACTK_STATS_INT_MAX, 0},
   
   { ACTK_STATS_USER_MALLOC,        "user-malloc",         ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_USER_FREE,          "user-free",           ACTK_STATS_INT_SUM, 0},
   
   { ACTK_STATS_ALLOC_FRESH,        "alloc-fresh",         ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_ALLOC_REUSE,        "alloc-reuse",         ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_ALLOC_FREED,        "alloc-freed",         ACTK_STATS_INT_SUM, 0},

   { ACTK_STATS_PROP,               "prop"               , ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_PROPREC,            "prop-rec"           , ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_PROP_QUEUEPOP,      "prop-queue-pop"     , ACTK_STATS_INT_SUM, 0},
   { ACTK_STATS_PROP_NOP,           "prop-nop"           , ACTK_STATS_INT_SUM, 0},
  };  

actk_stats_t* actk__stats_init(actk_stats_t* stats) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    stats->entries[i].stat = &(actk_stats[i]);
    stats->entries[i].val  = actk_stats[i].initial_val;
  }
  return stats;
}

actk_stats_t* actk__stats_save(actk_stats_t* stats_save,
                               actk_stats_t* stats) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    stats_save->entries[i] = stats->entries[i];
  }
  return stats_save;
}

actk_stats_t* actk__stats_diff(actk_stats_t* stats_diff,
                               actk_stats_t* stats_1,
                               actk_stats_t* stats_2) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    const actk_stats_stat_t* stat = stats_1->entries[i].stat;
    assert(stat == stats_2->entries[i].stat);
    stats_diff->entries[i].stat = stat;

    switch(stat->type) {
    case ACTK_STATS_INT_SUM:
      stats_diff->entries[i].val = (stats_1->entries[i].val -
                                    stats_2->entries[i].val );
      break;
    case ACTK_STATS_INT_MAX:
      stats_diff->entries[i].val = -1;
      break;
    }
  }
  return stats_diff;
}

void actk__stats_print(actk_stats_t* stats, FILE* file) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    fprintf(file, "actk-stats: %20s = %llu\n",
            stats->entries[i].stat->name,
            stats->entries[i].val);
  }
}

void actk__stats_print_column_key(actk_stats_t* stats, int col0, FILE* file) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    fprintf(file, "# column %3d : %20s\n",
            i + col0, stats->entries[i].stat->name);
  }
}

void actk__stats_print_header(actk_stats_t* stats, int col0, FILE* file) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    fprintf(file, "%16d ", i + col0);
  }
}

void actk__stats_print_line(actk_stats_t* stats, FILE* file) {
  int i;
  for(i = 0; i < ACTK_STATS_COUNT; i++) {
    fprintf(file, "%16llu ", stats->entries[i].val);
  }
}


