/* Matthew Hammer */
/* <hammer@tti-c.org */

#include <stdio.h>

#ifndef __STATS_H__
#define __STATS_H__

#include "actk_types.h"

typedef enum {
  /* Closures */
  ACTK_STATS_CLOSURE_NEW,
  ACTK_STATS_CLOSURE_INIT,
  ACTK_STATS_CLOSURE_ENTER,
  ACTK_STATS_CLOSURE_REPLACE,
  ACTK_STATS_CLOSURE_INVALIDATE,
  /* Recctxs */
  ACTK_STATS_RECCTX_NEW,
  ACTK_STATS_RECCTX_FRESH,
  ACTK_STATS_RECCTX_REUSE,
  /* Memory usage */
  ACTK_STATS_TOTAL_MALLOC,
  ACTK_STATS_TOTAL_FREE,
  ACTK_STATS_TOTAL_MAXLIVE,
  ACTK_STATS_USER_MALLOC,
  ACTK_STATS_USER_FREE,
  /* Allocation (counts) */
  ACTK_STATS_ALLOC_FRESH,
  ACTK_STATS_ALLOC_REUSE,
  ACTK_STATS_ALLOC_FREED,
  /* Propagation */
  ACTK_STATS_PROP,
  ACTK_STATS_PROPREC,
  ACTK_STATS_PROP_QUEUEPOP,
  ACTK_STATS_PROP_NOP,
  /* COUNT: Always last */
  ACTK_STATS_COUNT,
} actk_stats_entryid_t;

typedef enum {
  ACTK_STATS_INT_SUM,
  ACTK_STATS_INT_MAX,
} actk_stats_type_t;

typedef unsigned long long actk_stats_val_t;

typedef struct actk_stats_stat_s {
  actk_stats_entryid_t id;
  const char*          name;
  actk_stats_type_t    type;
  actk_stats_val_t     initial_val;
} actk_stats_stat_t;

typedef struct actk_stats_entry_s {
  const actk_stats_stat_t*  stat;
  actk_stats_val_t          val;
} actk_stats_entry_t;

struct actk_stats_s {
  actk_stats_entry_t entries[ACTK_STATS_COUNT];
};

actk_stats_t* actk__stats_init(actk_stats_t* stats);
actk_stats_t* actk__stats_save(actk_stats_t* stats_save, actk_stats_t* stats);
actk_stats_t* actk__stats_diff(actk_stats_t* stats_diff, actk_stats_t* stats_1, actk_stats_t* stats_2);
void          actk__stats_print(actk_stats_t* stats, FILE* file);

/* ----- For GNUPlot-style data-files: ------- */

void          actk__stats_print_column_key(actk_stats_t* stats, int col0, FILE* file);
void          actk__stats_print_header(actk_stats_t* stats, int col0, FILE* file);
void          actk__stats_print_line(actk_stats_t* stats, FILE* file);

/* ---------------------------------------------------------------- */
#undef actk__stats_ok
#define actk__stats_ok(__stats, __id)                                \
  (__extension__({                                                   \
    assert((__stats)->entries[ACTK_STATS_##__id].stat->id ==         \
           ACTK_STATS_##__id); }))                                   \
/* ---------------------------------------------------------------- */
#undef  actk__stats_incr
#define actk__stats_incr(__stats, __id)                              \
  (__extension__({                                                   \
    actk__stats_ok(__stats, __id);                                   \
    (__stats)->entries[ACTK_STATS_##__id].val ++; }))                \
/* ---------------------------------------------------------------- */
#undef  actk__stats_add
#define actk__stats_add(__stats, __id, __val)                        \
  (__extension__({                                                   \
    actk__stats_ok(__stats, __id);                                   \
    (__stats)->entries[ACTK_STATS_##__id].val += __val; }))          \
/* ---------------------------------------------------------------- */
#undef  actk__stats_max
#define actk__stats_max(__stats, __id, __val)                        \
  (__extension__({                                                   \
    actk__stats_ok(__stats, __id);                                   \
    if((__stats)->entries[ACTK_STATS_##__id].val < __val) {          \
      (__stats)->entries[ACTK_STATS_##__id].val = __val; } }))
/* ---------------------------------------------------------------- */
#undef  actk__stats_get
#define actk__stats_get(__stats, __id)                               \
  (__extension__({                                                   \
    actk__stats_ok(__stats, __id);                                   \
    (__stats)->entries[ACTK_STATS_##__id].val; }))                   \
/* ---------------------------------------------------------------- */


#endif
