#ifndef __BST_H__
#define __BST_H__

#include "actk.h"
#include <stdio.h>

/**
 * \defgroup actk_stdlib_bst Binary Search Trees
 * \ingroup actk_stdlib
 * @{
 */

typedef struct bst_node_s {
  void* key;
  actk_modref_t* parent;
  actk_modref_t* left;
  actk_modref_t* right;
} bst_node_t;

typedef int (*bst_compare_t)(void* key1, void* key2);

typedef struct bst_s {
  actk_state_t* actk_state;
  bst_compare_t comp;
  actk_modref_t* root;
  int initverif;
} bst_t;

typedef void (*bst_print_t)(bst_t* bst, void* env, void* key, FILE* file);

void        bst_init(actk_state_t* st, bst_t* bst, bst_compare_t comp);

void*       bst_find(bst_t* bst, void* key);
int         bst_ismemb(bst_t* bst, void* key);
int         bst_insert(bst_t* bst, void* key);
int         bst_remove(bst_t* bst, void* key);
void        bst_print(bst_t* bst, void* env, bst_print_t print, FILE* file);

void        bst_initverif(bst_t* bst);
void        bst_verify(bst_t* bst);

/* Searching via bst_finger gives the user a "finger" into the tree to
   do further operations in O(1) */
bst_node_t* bst_finger(bst_t* bst, void* key);
bst_node_t* bst_fing_pred(bst_t* bst, bst_node_t* finger);
bst_node_t* bst_fing_succ(bst_t* bst, bst_node_t* finger);
int         bst_fing_remove(bst_t* bst, bst_node_t* finger);

/* @} */

#endif
