#include "modlist.h"
#include "geom2d.h"
#include "geom2d_examples.h"
#include "quickhull.h"


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* Wrappers for geometry functions */

static float*
pair_dist(actk_state_t* st, void* dummy, modlist_cell_t* c) {
  modlist_pair_t* pair = (modlist_pair_t*) c->head;
  geom2d_point_t*   p1 = (geom2d_point_t*) pair->fst;
  geom2d_point_t*   p2 = (geom2d_point_t*) pair->snd;  
  float*      floatbox = actk_alloc(st, sizeof(float), pair);

  *floatbox = geom2d_dist(p1, p2);
  return floatbox;
}

static float*
maxf(actk_state_t* st, void* dummy, float* f1, float* f2) {
  if(*f1 > *f2)
    return f1;
  else
    return f2;  
}

static float*
minf(actk_state_t* st, void* dummy, float* f1, float* f2) {
  if(*f1 < *f2)
    return f1;
  else
    return f2;  
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

ACTK_FUN(diameter, diameter_, ACTK_USEALLOCTABLE);

void diameter_(actk_state_t* st) {
  actk_fun_begin(st);
  modlist_t*     list = actk_fun_arg(st);
  actk_modref_t* dest = actk_fun_arg(st);

  actk_modref_t* hull = actk_modref(st);
  actk_modref_t* pairs = actk_modref(st);
  actk_modref_t* dists = actk_modref(st);  
  
  actk_call(st, quickhull,
            actk_pass(list),
            actk_pass(hull)
            );

  actk_call(st, modlist_cross,
            actk_pass(hull),
            actk_pass(hull),
            actk_pass(pairs)
            );

  actk_call(st, modlist_map,
            actk_read(pairs),
            actk_pass(dists),
            actk_pass(pair_dist),
            actk_pass(NULL),
            );

  actk_call(st, modlist_combine,
            actk_pass(dists),
            actk_pass(dest),
            actk_pass(maxf),
            actk_pass(NULL)
            );
  
  actk_fun_end(st);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

ACTK_FUN(distance, distance_, ACTK_USEALLOCTABLE);

void distance_(actk_state_t* st) {
  actk_fun_begin(st);
  modlist_t*     list1 = actk_fun_arg(st);
  modlist_t*     list2 = actk_fun_arg(st);
  actk_modref_t*  dest = actk_fun_arg(st);

  actk_modref_t* hull1 = actk_modref(st);
  actk_modref_t* hull2 = actk_modref(st);
  actk_modref_t* pairs = actk_modref(st);
  actk_modref_t* dists = actk_modref(st);  
  
  actk_call(st, quickhull,
            actk_pass(list1),
            actk_pass(hull1)
            );

  actk_call(st, quickhull,
            actk_pass(list2),
            actk_pass(hull2)
            );

  actk_call(st, modlist_cross,
            actk_pass(hull1),
            actk_pass(hull2),
            actk_pass(pairs)
            );

  actk_call(st, modlist_map,
            actk_read(pairs),
            actk_pass(dists),
            actk_pass(pair_dist),
            actk_pass(NULL),
            );

  actk_call(st, modlist_combine,
            actk_pass(dists),
            actk_pass(dest),
            actk_pass(minf),
            actk_pass(NULL)
            );
  
  actk_fun_end(st);
}
