#include "mergesort.h"
#include "modutil.h"

ACTK_FUN_SIG(mergesort_rec, "mergesort_rec", "",
             ACTK_FUN_ARG("test", bool,                 READ_LOC_OR_VAL),
             ACTK_FUN_ARG("l",    modlist_cell_t*,      CONST_VAL),
             ACTK_FUN_ARG("d",    actk_modref_t*,       CONST_DST),
             ACTK_FUN_ARG("pred", modlist_cell_pred_t*, ENVIR_VAL) );

ACTK_FUN(modlist_mergesort, mergesort_, ACTK_USEALLOCTABLE);
     
void mergesort_(actk_state_t* st) {
  actk_fun_begin(st);
  modlist_t*           list = actk_fun_arg(st);
  actk_modref_t*       dest = actk_fun_arg(st);
  modlist_cell_pred_t* pred = actk_fun_arg(st);
  
  actk_modref_t*    testlen = actk_modref(st, list);
  
  actk_call( st, modlist_testlen,
             actk_read(list),
             actk_pass(1),
             actk_pass(testlen));
  
  actk_call( st, mergesort_rec,
             actk_read(testlen),
             actk_pass(list),
             actk_pass(dest),
             actk_pass(pred) );
  actk_fun_end(st);
}

ACTK_FUN(mergesort_rec, mergesort_rec_,
         ACTK_USEALLOCTABLE | ACTK_USEMEMOTABLE );

void mergesort_rec_(actk_state_t* st) {
  actk_fun_begin(st);
  int           length_test = (int) actk_fun_arg(st);
  modlist_t*           list = actk_fun_arg(st);
  actk_modref_t*       dest = actk_fun_arg(st);
  modlist_cell_pred_t* pred = actk_fun_arg(st);

  if(length_test) {
    actk_call( st, modutil_copy,
               actk_read(list),
               actk_pass(dest) );
  }
  else {
    actk_modref_t* split_1 = actk_modref(st);
    actk_modref_t* split_2 = actk_modref(st);
    actk_modref_t* msort_1 = actk_modref(st);
    actk_modref_t* msort_2 = actk_modref(st);
    actk_modref_t* ltest_1 = actk_modref(st);
    actk_modref_t* ltest_2 = actk_modref(st);
    actk_hash_t*   hash    = actk_alloc(st, sizeof(actk_hash_t));

    actk_hash_init(hash, 2);    

    /* split */
    actk_call(st, modlist_split,
              actk_read(list),
              actk_pass(split_1),
              actk_pass(split_2),
              actk_pass((modlist_cell_pred_t) actk_hash_ptr),
              actk_pass(hash));

    /* testlengths of split results */
    actk_call(st, modlist_testlen,
              actk_read(split_1),
              actk_pass(1),
              actk_pass(ltest_1) );
    
    actk_call(st, modlist_testlen,
              actk_read(split_2),
              actk_pass(1),
              actk_pass(ltest_2) );
    /* recur */
    actk_call(st, mergesort_rec,
              actk_read(ltest_1),
              actk_pass(split_1),
              actk_pass(msort_1) );
                  
    actk_call(st, mergesort_rec,
              actk_read(ltest_2),
              actk_pass(split_2),
              actk_pass(msort_2) );

    /* merge */
    actk_call(st, modlist_merge,
              actk_read(msort_1),
              actk_read(msort_2),
              actk_pass(dest),
              actk_pass(pred) );
  }
  actk_fun_end(st);
}
