#include "bst.h"

#include "test_apps.h"
#include "test_params.h"
#include "test_state.h"
#include "test_hooks.h"

/* BST test implementation */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
static bst_t bst;
static int* keys_insertorder;
static int* keys_updateorder;

static int int_comp(void* x, void* y) {
  if((int)x < (int)y) return -1;
  else if((int) y < (int)x) return 1;
  else return 0;
}

static void int_print(bst_t* bst, void* env, void* key, FILE* file) {
  fprintf(file, "%d", (int) key);
}

static void oneton(int datac, void** datav) {
  int i;
  for(i = 0; i < datac; i++)
    datav[i] = (void*) i;
}

static int rand_int(int lo, int hi) {
  return (rand() % (hi - lo)) + lo;
}

static void shuffle(int datac, void** datav) {
  int i;
  for(i = 0; i < datac - 1; i++) {
    int j = rand_int(i, datac - 1);
    void* temp = datav[i];
    datav[i] = datav[j];
    datav[j] = temp;
  }
}

void test_bst_do_gen_input(actk_state_t* st,
                           test_params_t* test_params,
                           test_state_t* test_st)
{
  int n = test_params->input_size;
  bst_init(st, &bst, int_comp);

  /* Generate a random insert order */
  keys_insertorder = malloc(n * sizeof(void*));
  oneton(n, (void**) keys_insertorder);
  shuffle(n, (void**) keys_insertorder);

  /* Generate a random update (remove/insert) order */
  keys_updateorder = malloc(n * sizeof(void*));
  oneton(n, (void**) keys_updateorder);
  shuffle(n, (void**) keys_updateorder);

  /* Generate the random initial (unverified) tree */
  {int i; for(i = 0; i < n; i++) {
    int key = keys_insertorder[i];
    bst_insert(&bst, (void*) key);
  }}
}

void test_bst_print_input(actk_state_t* st,
                          test_params_t* test_params,
                          test_state_t* test_st, FILE* file) {
  fprintf(file, "update key: %d", keys_updateorder[test_st->change_index]);
}

void test_bst_print_output(actk_state_t* st,
                           test_params_t* test_params,
                           test_state_t* test_st, FILE* file) {
  bst_print(&bst, NULL, int_print, file);
}

void test_bst_do_initial_run(actk_state_t* st,
                             test_params_t* test_params,
                             test_state_t* test_st) {
  bst_initverif(&bst);
}

void test_bst_do_prechange(actk_state_t* st,
                           test_params_t* test_params,
                           test_state_t* test_st)
{
  /* Nothing. */
}


void test_bst_do_insert(actk_state_t* st,
                        test_params_t* test_params,
                        test_state_t* test_st)
{
  /* This actually does a remove -- it will be re-inserted during the
     "remove" step. */
  int key = keys_updateorder[test_st->change_index];
  bst_remove(&bst, (void*) key);  
}

void test_bst_do_remove(actk_state_t* st,
                        test_params_t* test_params,
                        test_state_t* test_st)
{
  /* This actually does an insert -- it was removed during the
     "insert" step. */
  int key = keys_updateorder[test_st->change_index];
  bst_insert(&bst, (void*) key);
}

void test_bst_do_replace(actk_state_t* st,
                         test_params_t* test_params,
                         test_state_t* test_st)
{
  /* Replacement is not supported. */
  abort();
}

test_hooks_t test_hooks_bst = {
  test_bst_do_gen_input,
  test_bst_do_initial_run,
  test_bst_do_prechange,
  test_bst_do_replace,
  test_bst_do_insert,
  test_bst_do_remove,  
  test_bst_print_input,
  test_bst_print_output,
};
