#include "actk.h"
#include "modlist.h"
#include "quicksort.h"
#include "mergesort.h"
#include "geom2d.h"
#include "geom2d_examples.h"
#include "quickhull.h"


#include "test_apps.h"
#include "test_params.h"
#include "test_state.h"
#include "test_hooks.h"
#include "test_utils.h"

#include <math.h>

#define POINT2D_WRITE_INPUTOUTPUT 0

#define CASEAPP(test_params, appname) (!strcmp((test_params)->app->name, appname))

/* Utilities for (unboxed) integers */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
static int int_rand()
     /*{ return 1 + (int) (100 * (rand() / (RAND_MAX + 1.0))); }  */
{ return rand(); }

static int int_add(actk_state_t* st, actk_val_t dummy, int i1, int i2) {
  return i1 + i2;
}

static int int_min(actk_state_t* st, actk_val_t dummy, int i1, int i2) {
  return (i1 < i2 ? i1 : i2);
}

static void int_print(actk_state_t* st, FILE* f, int i) {
  fprintf(f, "%d", i);
}

static void intpair_print(actk_state_t* st, FILE* f, modlist_pair_t* p) {
  int i1 = (int) p->fst;
  int i2 = (int) p->snd;
  fprintf(f, "(%d, %d)", i1, i2);
}

static actk_val_t rand_int_generator(actk_state_t* st, actk_val_t i, actk_val_t* next_i) {
  *next_i = i - 1;
  return (actk_val_t) int_rand(st);
}

/* Utilities for (unboxed) integer mod-lists */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static actk_val_t cell_isodd (actk_state_t* st, actk_val_t env, modlist_cell_t* c) {
  return (actk_val_t) ((int)c->head % 2);
}

static actk_val_t cell_iseven (actk_state_t* st, actk_val_t env, modlist_cell_t* c) {
  return (actk_val_t) !((int)c->head % 2);
}

static actk_val_t cell_add (actk_state_t* st, actk_val_t a, modlist_cell_t* c) {
  return (actk_val_t) ((int) c->head) + ((int) a);
}

static actk_val_t cell_le_cell (actk_state_t* st, modlist_cell_t* c1, modlist_cell_t* c2) {
  return (actk_val_t) (c1->head <= c2->head);
}

static actk_val_t cell_le_int (actk_state_t* st, int x, modlist_cell_t* c) {
  return (actk_val_t) ((int) c->head <= x);
}

/* Utilities for (boxed) integers */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static int* boxint_rand(actk_state_t* st) {
  int* rand_int_ptr = actk_alloc(st, sizeof(int));
  *rand_int_ptr = int_rand();
  return rand_int_ptr;
}

static int* boxint_min(actk_state_t* st, actk_val_t dummy, int* i1, int* i2) {
  if(*i1 < *i2) return i1; else return i2;
}

static int* boxint_add(actk_state_t* st, actk_val_t dummy, int* i1, int* i2) {
  /* TODO: this shouldn't be keyed, but it needs to be for stability
     for sum benchmark: */
  int* box = actk_alloc(st, sizeof(int), i2); 
  *box = *i1 + *i2;;
  return box;
}

static void boxint_print(actk_state_t* st, FILE* f, int* i) {
  fprintf(f, "%d", *i);
}

static void boxintpair_print(actk_state_t* st, FILE* f, modlist_pair_t* p) {
  int* i1 = (int*) p->fst;
  int* i2 = (int*) p->snd;
  fprintf(f, "(%d, %d)", *i1, *i2);
}

/* Utilities for (boxed) floats */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static void boxfloat_print(actk_state_t* st, FILE* file, float* f) {
  fprintf(file, "%f", *f);
}

/* Utilities for 2D-points */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

static float
randf(float min, float max) {
  return (max - min) * ((float) rand() / (RAND_MAX + 1.0)) + min;
}

static float
randf_gaussian() {
  const float PI = 3.141521;
  float a = randf(0.0001, 0.9999);
  float b = randf(0.0001, 0.9999);
  float log_a = logf(a);
  float c = sqrtf(-2.0 * log_a) * cos(2.0 * PI * b);
  return c;
}

static float
randf_uniform() {
  return randf(-0.9999, 0.9999);
}

static geom2d_point_t* point2d_rand(actk_state_t* st) {
  geom2d_point_t* p = actk_alloc(st, sizeof(geom2d_point_t));
  p->x = randf_uniform();
  p->y = randf_uniform();
  return p;
}

static geom2d_point_t* point2d_rand2(actk_state_t* st) {
  geom2d_point_t* p = actk_alloc(st, sizeof(geom2d_point_t));
  p->x = randf_uniform() + 1.0001;
  p->y = randf_uniform() + 1.0001;
  return p;
}

static void point2d_print(actk_state_t* st, FILE* f, geom2d_point_t* p) {
  fprintf(f, "(%.2f, %.2f)", p->x, p->y);  
}

static actk_val_t rand_point2d_generator(actk_state_t* st, actk_val_t i, actk_val_t* next_i) {
  *next_i = i - 1;
  return (actk_val_t) point2d_rand(st);
}

static actk_val_t rand2_point2d_generator(actk_state_t* st, actk_val_t i, actk_val_t* next_i) {
  *next_i = i - 1;
  return (actk_val_t) point2d_rand2(st);
}

static void point2d_printlist(actk_state_t* st, FILE* f, actk_modref_t* list) {
  modlist_cell_t* c = (modlist_cell_t*) actk_deref(st, list);
  while(c) {
    geom2d_point_t* point = (geom2d_point_t*) c->head;
    fprintf(f, "%f %f\n", point->x, point->y);
    c = (modlist_cell_t*) actk_deref(st, c->tail);
  }
}

static void poind2d_datafile(actk_state_t* st, const char* filename, actk_modref_t* list) {
  if(POINT2D_WRITE_INPUTOUTPUT) {
    FILE* f = fopen(filename, "w+");
    point2d_printlist(st, f, list);
    fclose(f);
  }
}

/* Mod-list test implementation */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

void test_modlist_do_gen_input(actk_state_t* st,
                               test_params_t* test_params,
                               test_state_t* test_st)
{
  
  if(CASEAPP(test_params, "quickhull") ||
     CASEAPP(test_params, "diameter")) {
    
    test_st->input[0] = modlist_generate(st, rand_point2d_generator,
                                         (actk_val_t) test_params->input_size,
                                         (actk_val_t) 0);
    
  }
  else if(CASEAPP(test_params, "distance")) {

    test_st->input[0] = modlist_generate(st, rand_point2d_generator,
                                         (actk_val_t) (test_params->input_size / 2),
                                         (actk_val_t) 0);
    
    test_st->input[1] = modlist_generate(st, rand2_point2d_generator,
                                         (actk_val_t) (test_params->input_size / 2),
                                         (actk_val_t) 0);
  }
  else if(CASEAPP(test_params, "merge")) {    
    test_st->input[0] = modlist_generate(st, rand_int_generator,
                                         (actk_val_t) (test_params->input_size / 2),
                                         (actk_val_t) 0);

    test_st->input[1] = modlist_generate(st, rand_int_generator,
                                         (actk_val_t) (test_params->input_size / 2),
                                         (actk_val_t) 0);
  }
  else {
    test_st->input[0] = modlist_generate(st, rand_int_generator,
                                         (actk_val_t) test_params->input_size,
                                         (actk_val_t) 0);
  }
}

void test_modlist_print_input(actk_state_t* st,
                              test_params_t* test_params,
                              test_state_t* test_st, FILE* file) {

  if(CASEAPP(test_params, "quickhull") ||
     CASEAPP(test_params, "diameter"))
  {
    modlist_print(st, file, test_st->input[0], (modlist_val_printf_t) point2d_print);
  }
  else if(CASEAPP(test_params, "distance")) {
    modlist_print(st, file, test_st->input[0], (modlist_val_printf_t) point2d_print);
    modlist_print(st, file, test_st->input[1], (modlist_val_printf_t) point2d_print);
  }
  else if(CASEAPP(test_params, "merge")) {
    modlist_print(st, file, test_st->input[0], (modlist_val_printf_t) int_print);
    modlist_print(st, file, test_st->input[1], (modlist_val_printf_t) int_print);
  }  
  else {
    modlist_print_hi(st, file, test_st->input[0], (modlist_val_printf_t) int_print,
                     test_st->change_index, test_st->change_index);
  }
}

void test_modlist_print_output(actk_state_t* st,
                               test_params_t* test_params,
                               test_state_t* test_st, FILE* file) {  
    
  if(CASEAPP(test_params, "quicksort") ||
     CASEAPP(test_params, "mergesort") ||
     CASEAPP(test_params, "filter") ||
     CASEAPP(test_params, "merge") ||
     CASEAPP(test_params, "traverse") ||
     CASEAPP(test_params, "reverse") ||
     CASEAPP(test_params, "map"))
  {     
    modlist_print(st, file, test_st->output[0], (modlist_val_printf_t) int_print);
  }
  else if(CASEAPP(test_params, "cross"))
  {
    modlist_print(st, file, test_st->output[0], (modlist_val_printf_t) boxintpair_print);
  }  
  else if(CASEAPP(test_params, "minimum") ||
          CASEAPP(test_params, "sum"))
  {
    int_print(st, file, (int) actk_deref(st, test_st->output[0]));
  }
  else if(CASEAPP(test_params, "split")) {
    modlist_print(st, file, test_st->output[0], (modlist_val_printf_t) int_print);
    modlist_print(st, file, test_st->output[1], (modlist_val_printf_t) int_print);
  }
  else if(CASEAPP(test_params, "diameter") ||
          CASEAPP(test_params, "distance"))
  {
    boxfloat_print(st, file, (float*) actk_deref(st, test_st->output[0]));
  }
  else if(CASEAPP(test_params, "quickhull"))
  {
    modlist_print(st, file, test_st->output[0], (modlist_val_printf_t) point2d_print);
    poind2d_datafile(st, cheap_sprintf("pnts-%02d.data", test_st->change_count), test_st->input[0]);
    poind2d_datafile(st, cheap_sprintf("hull-%02d.data", test_st->change_count), test_st->output[0]);
  }
  else {
    abort();
  }
}

void test_modlist_do_initial_run(actk_state_t* st,
                                 test_params_t* test_params,
                                 test_state_t* test_st) {
  if(CASEAPP(test_params, "quicksort")) {
    actk_call(st, quicksort_via_split,
              actk_read(test_st->input[0]),
              actk_pass(modlist_nil_cell),
              actk_pass(test_st->output[0]),
              actk_pass(cell_le_int)
              );
  }
  else if(CASEAPP(test_params, "mergesort")) {
    actk_call(st, modlist_mergesort,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->output[0]),
              actk_pass(cell_le_cell) );
  }
  else if(CASEAPP(test_params, "filter")) {
    actk_call(st, modlist_filter,
              actk_read(test_st->input[0]),
              actk_pass(test_st->output[0]),
              actk_pass(cell_isodd),
              actk_pass(NULL));
  }
  else if(CASEAPP(test_params, "map")) {
    actk_call(st, modlist_map,
              actk_read(test_st->input[0]),
              actk_pass(test_st->output[0]),
              actk_pass(cell_add),
              actk_pass(23));
  }
  else if(CASEAPP(test_params, "merge")) {
    actk_call(st, modlist_merge,
              actk_read(test_st->input[0]),
              actk_read(test_st->input[1]),
              actk_pass(test_st->output[0]),
              actk_pass(cell_le_int)
              );
  }  
  else if(CASEAPP(test_params, "split")) {
    static actk_hash_t hash;
    actk_hash_init(&hash, 2);
    
    actk_call(st, modlist_split,
              actk_read(test_st->input[0]),
              actk_pass(test_st->output[0]),
              actk_pass(test_st->output[1]),
              actk_pass((modlist_cell_pred_t) actk_hash_ptr),
              actk_pass(&hash));
  } 
  else if(CASEAPP(test_params, "minimum")) {
    actk_call(st, modlist_combine,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->output[0]),
              actk_pass(int_min),
              actk_pass(NULL));
  }
  else if(CASEAPP(test_params, "sum")) {
    actk_call(st, modlist_combine,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->output[0]),
              actk_pass(int_add),
              actk_pass(NULL));    
  }
  else if(CASEAPP(test_params, "reverse")) {
    actk_call(st, modlist_reverse_rec,
              actk_read(test_st->input[0]),
              actk_pass(modlist_nil_cell),
              actk_pass(test_st->output[0]));
  }
  else if(CASEAPP(test_params, "cross")) {
    actk_call(st, modlist_cross,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->input[0]),
              actk_pass(test_st->output[0]));
  }
  else if(CASEAPP(test_params, "traverse")) {
    actk_write(st, test_st->output[0], (actk_val_t) modlist_nil_cell);
    actk_call(st, modlist_traverse,
              actk_read(test_st->input[0])
              );
  }
  else if(CASEAPP(test_params, "quickhull")) {
    actk_call(st, quickhull,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->output[0])
              );
    poind2d_datafile(st, "hull.data", test_st->output[0]); 
  }
  else if(CASEAPP(test_params, "diameter")) {
    actk_call(st, diameter,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->output[0])
              );
  }
  else if(CASEAPP(test_params, "distance")) {
    actk_call(st, distance,
              actk_pass(test_st->input[0]),
              actk_pass(test_st->input[1]),
              actk_pass(test_st->output[0])
              );
  }
  else {
    const char* app_name = test_params->app->name;
    fprintf(stderr, "No test here for this: %s\n", app_name);
  }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

void test_modlist_do_prechange(actk_state_t* st,
                               test_params_t* test_params,
                               test_state_t* test_st) {
  test_st->change_modref =
    modlist_get_nth(st, test_st->change_modref,
                    test_st->change_index_delta);

  if(!test_st->change_modref) {

    if(( CASEAPP(test_params, "distance") ||
         CASEAPP(test_params, "merge")
         ) && test_st->change_phase == 0 )
    {
      test_st->change_modref = test_st->input[1];
      test_st->change_phase = 1;
    }
    else {
      abort();
    }
  }
}

void test_modlist_do_remove(actk_state_t* st,
                            test_params_t* test_params,
                            test_state_t* test_st) {
  actk_val_t val = modlist_remove(st, test_st->change_modref);

  /* Identify the garbage: */
  if(CASEAPP(test_params, "quickhull") ||
     CASEAPP(test_params, "diameter") ||
     CASEAPP(test_params, "distance"))
  {
    actk_garbage(st, (void*) val);
  }
  else {
    /* Unboxed ints ==> no garbage here */
  }
}

void test_modlist_do_replace(actk_state_t* st,
                             test_params_t* test_params,
                             test_state_t* test_st) {
  
  if(CASEAPP(test_params, "quickhull") ||
     CASEAPP(test_params, "diameter") ||
     CASEAPP(test_params, "distance"))
  {
    modlist_replace(st, test_st->change_modref, (actk_val_t) point2d_rand(st));
  }
  else
  {
    modlist_replace(st, test_st->change_modref, (actk_val_t) int_rand(st));
  }
}

void test_modlist_do_insert(actk_state_t* st,
                            test_params_t* test_params,
                            test_state_t* test_st) {
    
  if(CASEAPP(test_params, "quickhull") ||
     CASEAPP(test_params, "diameter"))
  {
    modlist_insert(st, test_st->change_modref, (actk_val_t) point2d_rand(st));
  }
  else if(CASEAPP(test_params, "distance")) {

    if(test_st->change_phase == 0)
      modlist_insert(st, test_st->change_modref, (actk_val_t) point2d_rand(st));
    
    else if(test_st->change_phase == 1)
      modlist_insert(st, test_st->change_modref, (actk_val_t) point2d_rand2(st));

    else
      abort();
  }
  else
  {
    modlist_insert(st, test_st->change_modref, (actk_val_t) int_rand(st));
  }
}

test_hooks_t test_hooks_modlist = {
  test_modlist_do_gen_input,
  test_modlist_do_initial_run,
  test_modlist_do_prechange,
  test_modlist_do_replace,
  test_modlist_do_insert,
  test_modlist_do_remove,  
  test_modlist_print_input,
  test_modlist_print_output,
};
