#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "actk.h"

#include "test_params.h"
#include "test_state.h"
#include "test_apps.h"
#include "test_hooks.h"

const char* test_flagval_name(test_flagval_t flagval) {
  static const char* flagval_names[] = {
    "(first)", "off", "on", "force-off", "force-on", "(last)"
  };
  assert(flagval > TEST_FLAGVAL__FIRST);
  assert(flagval < TEST_FLAGVAL__LAST);
  return flagval_names[flagval];
}

int test_flagval(test_flagval_t flagval) {
  return (flagval == TEST_FLAGVAL_ON ||
          flagval == TEST_FLAGVAL_FORCE_ON);
}

const char* test_change_name(test_change_t change) {
  static const char* change_names[] = {
    "(first)", "replace", "insert", "remove", "(last)"
  };
  assert(change > TEST_CHANGE__FIRST);
  assert(change < TEST_CHANGE__LAST);
  return change_names[change];
}

void test_params_print(test_params_t* test_params, FILE* f) {
  const char* fmt_int = "test-params: %15s = %d\n";
  const char* fmt_flg = "test-params: %15s = %s\n";
  const char* fmt_str = "test-params: %15s = %s\n";

  fprintf(f, fmt_str, "app",               test_params->app ? test_params->app->name : "NONE" );
  fprintf(f, fmt_int, "rand_seed",         test_params->rand_seed);
  fprintf(f, fmt_int, "input-size",        test_params->input_size);
  fprintf(f, fmt_int, "full-loops",        test_params->full_loops);
  fprintf(f, fmt_int, "change-loops",      test_params->change_loops);
  fprintf(f, fmt_int, "change-skip",       test_params->change_skip);
  fprintf(f, fmt_int, "change-limit",      test_params->change_limit);
  fprintf(f, fmt_flg, "do-replace",        test_flagval_name(test_params->change_do_replace));
  fprintf(f, fmt_flg, "do-ins-rem",        test_flagval_name(test_params->change_do_ins_rem));
  fprintf(f, fmt_flg, "print-inout",       test_flagval_name(test_params->print_inout));
  fprintf(f, fmt_flg, "write-stats",       test_flagval_name(test_params->write_stats));
  fprintf(f, fmt_flg, "write-timestamps",  test_flagval_name(test_params->write_timestamps));
}

void test_params_default(test_params_t* test_params) {
  test_params->app               = NULL;
  test_params->rand_seed         = 0;
  test_params->input_size        = 10;
  test_params->change_loops      = 1;
  test_params->full_loops        = 1;
  test_params->change_skip       = 1;
  test_params->change_limit      = -1;
  test_params->change_do_replace = TEST_FLAGVAL_OFF;
  test_params->change_do_ins_rem = TEST_FLAGVAL_ON;
  test_params->print_inout       = TEST_FLAGVAL_ON;
  test_params->write_stats       = TEST_FLAGVAL_OFF;
  test_params->write_timestamps  = TEST_FLAGVAL_OFF;
}

int test_params__get_argv_int(char** argv, int argc, int argi) {
  if(argi >= argc) {
    fprintf(stderr, "expected an integer after %s\n", argv[argi-1]);
    exit(-1);
  }    
  return atoi(argv[argi]);
}

char* test_params__get_argv_string(char** argv, int argc, int argi) {
  if(argi >= argc) {
    fprintf(stderr, "expected a string after %s\n", argv[argi-1]);
    exit(-1);
  }    
  return argv[argi];
}

int test_params_from_argv(test_params_t* test_params,
                          int argc, char** argv) {
  int i;
  for(i = 1; i < argc; i++) {
    char* arg = argv[i];
    if(!strncmp("-actk", arg, 5) ||
       !strncmp("+actk", arg, 5)) {
      return i;
    }
    if(!strcmp("-app", arg)) {
      char* name = test_params__get_argv_string(argv, argc, ++i);
      test_app_t* app = test_app_from_name(name);
      if(!app) {
        fprintf(stderr, "no app with name: %s\n", name);
        exit(EXIT_FAILURE);
      }
      test_params->app = app;
    }
    else if(!strcmp("-input-size", arg)) {
      test_params->input_size =
        test_params__get_argv_int(argv, argc, ++i);
      if(test_params->input_size > 20 &&
         test_params->print_inout == TEST_FLAGVAL_ON)
        test_params->print_inout = TEST_FLAGVAL_OFF;
    }
    else if(!strcmp("-print-inout", arg)) {
      test_params->print_inout = TEST_FLAGVAL_FORCE_ON;
    }
    else if(!strcmp("+print-inout", arg)) {
      test_params->print_inout = TEST_FLAGVAL_FORCE_OFF;
    }
    else if(!strcmp("-write-stats", arg)) {
      test_params->write_stats = TEST_FLAGVAL_FORCE_ON;
    }
    else if(!strcmp("+write-stats", arg)) {
      test_params->write_stats = TEST_FLAGVAL_FORCE_OFF;
    }
    else if(!strcmp("-write-timestamps", arg)) {
      test_params->write_timestamps = TEST_FLAGVAL_FORCE_ON;
    }
    else if(!strcmp("+write-timestamps", arg)) {
      test_params->write_timestamps = TEST_FLAGVAL_FORCE_OFF;
    }
    else if(!strcmp("-change-loops", arg)) {
      test_params->change_loops =
        test_params__get_argv_int(argv, argc, ++i);
    }
    else if(!strcmp("-full-loops", arg)) {
      test_params->full_loops =
        test_params__get_argv_int(argv, argc, ++i);
    }
    else if(!strcmp("-change-skip", arg)) {
      test_params->change_skip =
        test_params__get_argv_int(argv, argc, ++i);
    }
    else if(!strcmp("-change-limit", arg)) {
      test_params->change_limit =
        test_params__get_argv_int(argv, argc, ++i);
    }
    else if(!strcmp("-do-replace", arg)) {
      test_params->change_do_replace = TEST_FLAGVAL_FORCE_ON;

      if(test_params->change_do_ins_rem == TEST_FLAGVAL_ON)
        test_params->change_do_ins_rem = TEST_FLAGVAL_OFF;
     }
    else if(!strcmp("-do-ins-rem", arg)) {
      test_params->change_do_ins_rem = TEST_FLAGVAL_FORCE_ON;

      if(test_params->change_do_replace == TEST_FLAGVAL_ON)
        test_params->change_do_replace = TEST_FLAGVAL_OFF;
    }
    else if(!strcmp("-srand", arg)) {
      int seed = test_params__get_argv_int(argv, argc, ++i);

      if(seed == 0) {
        struct timeval tv;
        gettimeofday(&tv, NULL);
        seed = tv.tv_usec;
      }
      
      srand(seed);
      test_params->rand_seed = seed;
    }
    else if(!strcmp("-perf-mode", arg)) {
      test_params->print_inout = TEST_FLAGVAL_FORCE_OFF;
    }
    else {
      fprintf(stderr, "test: ERR: argument not recognized: %s\n", arg);
      exit(EXIT_FAILURE);
    }
  }
  return i;
}

