#include "test_utils.h"
#include <stdlib.h>
#include <stdarg.h>

char* cheap_sprintf(const char* fmt, ...) {
#define BUFFSZ 127
  static char buff[BUFFSZ + 1];
  va_list ap;
  va_start(ap, fmt);
  vsnprintf(buff, BUFFSZ, fmt, ap);
  va_end(ap);
  return buff;
}

char* strapp2(const char* s1, const char* s2) {
  return cheap_sprintf("%s%s", s1, s2);
}

char* strapp3(const char* s1, const char* s2, const char* s3) {
  return cheap_sprintf("%s%s%s", s1, s2, s3);
}
  
void print_size(FILE* f, const char* label, long long size) {
  const long long _1K = 1 << 10;
  const long long _1M = 1 << 20;
  const long long _1G = 1 << 30;

  char* suffix = "";
  float coeff = 0;
  
  if(size < _1K) {
    coeff = size;
    suffix = "";
  }
  else if(size < _1M) {
    coeff = (float)size / (float)_1K;
    suffix = "K";
  }
  else if(size < _1G) {
    coeff = (float)size / (float)_1M;
    suffix = "M";
  }
  else {
    coeff = (float)size / (float)_1G;
    suffix = "G";
  }
  
  printf("%s = %llu (%.1f%s)\n",
         label, size,
         coeff, suffix);  
}
