exception Bad_algorithm
open Util
open Printf

type command_in = { algorithm    : string;
                    input_size   : int;
                    bookkeeping  : bool
                  }

module String_map = Map.Make(String)

type command_out = float String_map.t

let to_string (cin : command_in)  = 
  let binary = Experiments.binary_path in
  let application = (sprintf "-app %s" cin.algorithm) in
  let bookkeeping = 
    if cin.bookkeeping 
    then "-actk-bookkeeping"
    else "+actk-bookkeeping"
  in
  (sprintf "%s -perf-mode -srand 0 %s -input-size %d %s 2>/dev/null"
     binary
     application
     cin.input_size
     bookkeeping)

let run (cmd : string) (msg: string) =
  let _    = eprintf "%s: %s: running: %s ...\n%!" myname msg cmd in
  let inch = Unix.open_process_in cmd in
  let out = (String_map.empty : command_out) in
  (* regexp matches key/val pairs of the form: KEY = FLOAT-VALUE *)
  let key_regexp = "\\([-a-zA-Z]+\\)" in
  let val_regexp = "\\(-?[0-9]+\\(\\.[0-9]+\\([eE][+-][0-9]+\\)?\\)?\\)" in
  let regexp = Str.regexp (sprintf "[ \t]*%s[ \t]*=[ \t]*%s" key_regexp val_regexp) in
  let rec loop out =
    try
      let line = (input_line inch) in
      let succ = Str.string_match regexp line 0 in
      if (succ) then
        let key    = Str.matched_group 1 line in
        let valstr = Str.matched_group 2 line in
        let valflt = float_of_string valstr in
       (* let _ = eprintf "%s = %s = %f\n" key valstr valflt in *)
        loop (String_map.add key valflt out)
      else
        loop out
    with
        End_of_file -> (ignore (Unix.close_process_in inch); out)
  in (loop out)
