open Printf

type input = { algo_name   : string ;
               input_size  : int ;
               num_samples : int }
    
type output = { sample_ave : Sample.output ;
                samples    : Sample.output list }

type t = input * output

exception User_quit of t

let fresh input =
  (input, {sample_ave = Sample.ave_none; samples = []})

let resume entry =
  let (input, output) = entry in
  let rec loop sample_num samples =
    try
      if sample_num < input.num_samples then        
        let sample_in  = { Sample.algo_name = input.algo_name ;
                           Sample.input_size = input.input_size ;
                           Sample.sample_num = sample_num 
                         } 
        in
        let _ = Unix.sigprocmask Unix.SIG_UNBLOCK [Sys.sigint] in
        let sample_out = Sample.take sample_in in
        let _ = Unix.sigprocmask Unix.SIG_BLOCK [Sys.sigint] in
        loop (sample_num + 1) (sample_out::samples)
      else
        samples
    with
        Util.User_quit -> 
          raise (User_quit 
                   (input, {sample_ave = Sample.ave_none; 
                            samples = samples } ))
  in
  let samples     = (loop (List.length output.samples) output.samples) in
  let samples_sum = Sample.combine samples (fun v1 v2 -> v1 +. v2) in
  let divisor     = (float_of_int input.num_samples) in
  let samples_ave = Sample.map (fun v -> v /. divisor) samples_sum in
  (input, { sample_ave = samples_ave; 
            samples = samples })
