open Printf

(* Data-file column numbers, starting with 1 *)
let c_inputsize   = 1
let c_static_time = 2
let c_self_time   = 3
let c_update_time = 4

type plotscript_params = { appname : string ;
                           datadir : string ;
                           apptitle : string ;
                           mldatadir : string option ;
                           xtics : int
                         }

let plotscript params = 
  let buffer = Buffer.create 1000 in  
  let add s = (Buffer.add_string buffer s) in  
  let anl s = add s; add "\n" in
  let output_dir = "." in
  let datafile = (sprintf "%s/%s.data" output_dir params.appname) in
  
  let (mldatadir, useml) = match params.mldatadir with 
      Some x -> (x, true) 
    | None -> ("NONE", false)
  in

  (* Some constants: data-file paths and plot labels *)
  let path_static = "../data_20080128/" ^ params.appname ^ ".data" in
  let path_sml_static = "../" ^ mldatadir ^ "/Verifier" in
  let path_sml_fromscratch = "../" ^ mldatadir ^ "/InitialRun" in
  let path_sml_changeprop = "../" ^ mldatadir ^ "/AllInsertDelete" in
  let path_sml_speedup = (sprintf "< paste %s/full.dat %s/app.dat" 
                           path_sml_changeprop path_sml_static ) in
  let label_sml = " SML" in
  let label_sml_with_gc = " SML+GC" in
  let label_sml_without_gc = " SML-GC" in
  let label_c_self = "C" in
  let label_c_static = " C,static" in  

  (* plotfile prologue: *)
  anl "set terminal postscript enhanced mono \"Helvetica\" 16" ;
  anl "set size 0.5,0.5" ;
  anl ("set xtics " ^ (string_of_int params.xtics)) ;
  anl "set xlabel \"Input Size ({/Times-Italic n} {/Symbol \\264} 10^3)\"";
  anl "set key left Right box" ;
  anl "set grid" ;  
  
  (* plot: From-scratch-Run *)
  anl "set ylabel \"Time (s)\"" ;
  anl (sprintf "set title \"%s From-Scratch\"" params.apptitle) ;
  anl (sprintf "set output \"%s/%s-fromscratch.ps\"" output_dir params.appname) ;
  anl (sprintf "plot \\") ;
  
  if useml then begin
    anl (sprintf "\"%s/full.dat\" using ($1/1000):2 t \"%s\" with linespoints,\\" 
           path_sml_fromscratch label_sml_with_gc) ;
    
    anl (sprintf "\"%s/app.dat\" using ($1/1000):2 t \"%s\" with linespoints,\\" 
           path_sml_fromscratch label_sml_without_gc) ;
  end ;
    
  anl (sprintf "\"%s\" using ($%d/1000):%d t \"%s\" with linespoints,\\" 
         datafile c_inputsize c_self_time label_c_self) ;
  
  anl (sprintf "\"%s\" using ($%d/1000):%d t \"%s\" with linespoints" 
         path_static c_inputsize c_static_time label_c_static) ;
    
  (* plot: Ave-Update *)
  anl "set ylabel \"Time (ms)\"" ;
  anl (sprintf "set title \"%s Ave. Update\"" params.apptitle) ;
  anl (sprintf "set output \"%s/%s-aveupdate.ps\"" output_dir params.appname) ;
  anl (sprintf "plot \\") ;
    
  if useml then begin
    anl (sprintf "\"%s/full.dat\" using ($1/1000):($2*1000) t \"%s\" with linespoints,\\"
           path_sml_changeprop label_sml_with_gc);
    
    anl (sprintf "\"%s/app.dat\" using ($1/1000):($2*1000) t \"%s\" with linespoints,\\"
           path_sml_changeprop label_sml_without_gc);  
  end ;
  
  anl (sprintf "\"%s\" using ($%d/1000):($%d*1000) t \"%s\" with linespoints" 
         datafile c_inputsize c_update_time label_c_self) ;

  (* plot: Speed-up *)
  (*
  anl "set ylabel \"Speed-up\"" ;
  anl (sprintf "set title \"%s Speedup\"" params.apptitle) ;
  anl (sprintf "set output \"%s/%s-speedup.ps\"" output_dir params.appname) ;
  anl (sprintf "plot \\") ;
  
  anl (sprintf "\"%s\" using ($%d/1000):($%d/$%d) t \"%s\" with linespoints,\\" 
         datafile c_inputsize c_static_time c_update_time label_c_self) ;
  
  if useml then begin
    anl (sprintf "\"%s\" using ($1/1000):($4/$2) title \"%s\" w linespoints"
           path_sml_speedup label_sml ) ;
  end ;
  *)
  
  (Buffer.contents buffer)


(* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *)
(* Default behavior *)

let _ = 
  (List.iter 
     (fun (appname, domain) -> 
        
        let mldatadir =
          match Util.mlappname appname with
              Some appname -> Some (Util.mldatadir ^ appname)
            | None -> None
        in
        
        let domain_max = List.hd (List.rev domain) in

        let params = { appname = appname;
                       apptitle = Experiments.apptitle appname;
                       datadir = Experiments.data_path;
                       mldatadir = mldatadir ;
                       xtics = ( if domain_max > 500000 then 500 
                                 else if domain_max = 300000 then 60
                                 else if domain_max = 100000 then 20 else 10 )
                     }
        in
        if List.length domain > 1 then        
          let ps = plotscript params in
          let _ = eprintf "Writing: %s/%s.gnuplot\n" params.datadir params.appname in
          let file = open_out (sprintf "%s/%s.gnuplot" params.datadir params.appname) in
          let _ = output_string file ps in
          let _ = close_out file in 
          () 
        else 
          let _ = eprintf "Skipping: %s\n" params.appname in
          ()
     )
     Experiments.applications) ; eprintf "done.\n"
