open Printf
open Scanf
open Util
  
(* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *)
(* Hard-coded stuff *)

let time_row_string floats (t1, t2, t3, t4, t5) =
  match floats with
      inputsize::
        statictime::
        selftime::
        aveupdate::
        maxlive::_
      ->        
        let row = 
          (sprintf "%s& %s&%s&%s&%s&  %s&%s&%s& %s&%s"
             (format_float_pow10 inputsize)
             
             (format_float_norm statictime)
             (format_float_norm selftime)
             (t2 (* ML+GC*))
             (t1 (* ML-GC*))

             
             (format_float_small aveupdate)
             (t4 (* ML+GC*))
             (t3 (* ML-GC*))

             (format_float_space maxlive)
             (t5 (* ML *))
          )
        in
        row
    | _ -> invalid_arg "too few floats given."

let overhead_row_string floats (t1, t2, t3, t4, t5) =
  match floats with
      inputsize::
        statictime::
        selftime::
        aveupdate::
        maxlive::_
      ->        
        (* input-size,
           from-scratch,
           initial,
           aveupdate,
           speedup,
           overhead_1,
           overhead_2,
        *)
        let row = 
          (*Columns: oh1 su*)
          (sprintf "%s& %s& %s"
             (format_float_pow10 inputsize)
             (format_float_norm (selftime /. statictime))
             (format_float_large (statictime /. aveupdate))
          )
        in
        row
    | _ -> invalid_arg "too few floats given."
        
let _ =
  let time_file  = open_out "table-time.tex" in
  let overhead_file  = open_out "table-overhead.tex" in
  
  (List.iter 
     (fun (appname, _) ->       
        
        let last_line = line_last (Experiments.data_path ^ "/" ^ appname ^ ".data") in
        
        let floats = line_to_floats last_line in
        
        let mldata = match (mlappname appname) with
            Some mlappname -> getmldata mlappname (List.hd floats)
          | None -> ("{\\it n/a}", "{\\it n/a}", "{\\it n/a}", "{\\it n/a}", "{\\it n/a}")
        in

        (* Create a row in the time file *)
        let time_row  = (sprintf "{\\bf \\tt %16s} & %s \\\\ \\hline\n" (appname) 
                           (time_row_string floats mldata)) in
        let _ = output_string time_file time_row in

        (* Create a row in the time file *)
        let overhead_row  = (sprintf "{\\bf \\tt %16s} & %s \\\\ \\hline\n" (appname) 
                               (overhead_row_string floats mldata)) in
        let _ = output_string overhead_file overhead_row in
        
        ()
     )
     Experiments.applications) ; eprintf "done.\n"
    
