open Util
open Printf

exception Time_out

type input = { algo_name : string ;
               input_size : int ;
               sample_num : int 
             }

module String_map = Map.Make(String)

type output = float String_map.t

type t = input * output

let rec string_map_combine maps op =  
  match maps with
    | [] -> invalid_arg "map_sum"
    | [map] -> map
    | map1::map2::maps -> 
        let map3 = (String_map.mapi
                      (fun key value -> op (String_map.find key map2) value) 
                      map1) in
        (string_map_combine (map3::maps) op)

let ave_none = String_map.empty

let combine = string_map_combine

let map = String_map.map

let get = String_map.find

let run_command input cmdstr keys =
  let rec loop tries = 
    let tries = tries + 1 in    
    let results = Command.run cmdstr 
      (sprintf "(sample %d, try %d)" input.sample_num tries) 
    in
    let pass = List.fold_left
      (fun pass key -> 
         try ignore (String_map.find key results) ; pass 
         with
             Not_found -> 
               eprintf "%s: ERROR: missing result-key on try %d: %s\n" myname tries key ;
               if tries < 5 then false
               else
                 raise Time_out
      ) true keys 
    in
    if pass then results else (loop tries)
  in
  (loop 0)

let take input =  
  let static_cmdstr = Command.to_string { 
    Command.algorithm    = input.algo_name;
    Command.input_size   = input.input_size;
    Command.bookkeeping  = false; } in
  
  let static_keys = ["initial-time"] in
  
  let static_results = 
    (run_command input static_cmdstr static_keys) in
  
  let static_time = (String_map.find "initial-time" static_results) in

  let selfadj_cmdstr = Command.to_string { 
    Command.algorithm    = input.algo_name;
    Command.input_size   = input.input_size;
    Command.bookkeeping  = true; } in
  
  let selfadj_keys = ["initial-time";
                      "ave-update-time" ;
                      "ave-closure-cost" ;
                      "ave-gc-cost" ;
                      "user-malloc" ;
                      "user-free" ;
                      "user-live" ;
                      "total-malloc" ;
                      "total-free"  ;
                      "total-live" ;
                      "total-maxlive" ;
                     ] in
  
  let selfadj_results =
    (run_command input selfadj_cmdstr selfadj_keys) in
  
  let results = 
    (String_map.add "static-time" static_time selfadj_results) in
  (results)
  
