open Util
open Printf

(* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *)
(* Setup the signal handler to dump a checkpoint if we want to quit  *)
let _ =
  let handle_user_interrupt signo =
    let _ = Unix.sigprocmask Unix.SIG_BLOCK [Sys.sigint] in
    eprintf "%s: got user interrupt\n%!" myname ;  
    let rec loop () = 
      let _ = printf "action? [(c)ontinue | (q)uit/save | (a)bort)]: %!" in
      let c = String.lowercase (input_line stdin) in
      match c with
          "c" -> eprintf "%s: continuing ...\n%!" myname
        | "a" -> exit(0)
        | "q" -> raise User_quit
        | _   -> loop ()
    in
    loop ()
  in
  let _ = Sys.set_signal Sys.sigint (Sys.Signal_handle handle_user_interrupt) in
  let _ = Unix.sigprocmask Unix.SIG_BLOCK [Sys.sigint] in
  ()

(* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *)
(* The argument parsing, etc                                       *)
let _ =
  let all _ = 
    let _ = (List.map
               (fun (appname, domain) ->
                  let table = Table.fresh appname domain 
                    Experiments.num_samples
                  in (ignore (Table.run table)))
               Experiments.applications)
    in exit(0)
  in
  let resume checkpt =
    let table = Table.load checkpt in
    let _     = Table.run table in
    exit(0)
  in    
  let args = [
    ("-all", Arg.Unit all, "run all available tests") ;
    ("-resume", Arg.String resume, "resume a test from a given checkpoint file")
  ]
  in
  let _ = Arg.parse 
    (Arg.align args) 
    (fun x -> 
       eprintf "%s: argument not recognized: %s\n" myname x; 
       Arg.usage args (sprintf "%s: usage:" myname) ;
       exit(1))
    ("testperf : a performance tester for the ACTK project\n" ^
       " Matthew Hammer <hammer@tti-c.org>") in
  (all ())
